@@ -85,6 +85,20 @@ class EddyParser(ArgumentParser):
             help="Levels : DEBUG, INFO, WARNING," " ERROR, CRITICAL",
         )
 
+    def memory_arg(self):
+        self.add_argument(
+            "--memory",
+            action="store_true",
+            help="Load file in memory before to read with netCDF library",
+        )
+
+    def contour_intern_arg(self):
+        self.add_argument(
+            "--intern",
+            action="store_true",
+            help="Use intern contour instead of outter contour",
+        )
+
     def parse_args(self, *args, **kwargs):
         logger = start_logger()
         # Parsing
@@ -118,7 +132,7 @@ VAR_DESCR = dict(
         nc_dims=("obs",),
         nc_attr=dict(
             long_name="Rotating sense of the eddy",
-            comment="Cyclonic -1; Anti-cyclonic +1",
+            comment="Cyclonic -1; Anticyclonic +1",
         ),
     ),
     segment_size=dict(
@@ -183,7 +197,7 @@ VAR_DESCR = dict(
         nc_attr=dict(
             units="degrees_east",
             axis="X",
-            comment="Longitude center of the fitted circle",
+            comment="Longitude center of the best fit circle",
             long_name="Eddy Center Longitude",
             standard_name="longitude",
         ),
@@ -200,7 +214,7 @@ VAR_DESCR = dict(
             axis="Y",
             long_name="Eddy Center Latitude",
             standard_name="latitude",
-            comment="Latitude center of the fitted circle",
+            comment="Latitude center of the best fit circle",
         ),
     ),
     lon_max=dict(
@@ -215,6 +229,7 @@ VAR_DESCR = dict(
             axis="X",
             long_name="Longitude of the SSH maximum",
             standard_name="longitude",
+            comment="Longitude of the inner contour",
         ),
     ),
     lat_max=dict(
@@ -229,6 +244,7 @@ VAR_DESCR = dict(
             axis="Y",
             long_name="Latitude of the SSH maximum",
             standard_name="latitude",
+            comment="Latitude of the inner contour",
         ),
     ),
     amplitude=dict(
@@ -254,7 +270,7 @@ VAR_DESCR = dict(
         nc_attr=dict(
             long_name="Speed area",
             units="m^2",
-            comment="Area enclosed by speed contour in m^2",
+            comment="Area enclosed by the speed contour in m^2",
         ),
     ),
     effective_area=dict(
@@ -265,7 +281,7 @@ VAR_DESCR = dict(
         nc_attr=dict(
             long_name="Effective area",
             units="m^2",
-            comment="Area enclosed by effective contour in m^2",
+            comment="Area enclosed by the effective contour in m^2",
         ),
     ),
     speed_average=dict(
@@ -293,7 +309,7 @@ VAR_DESCR = dict(
         nc_attr=dict(
             long_name="Radial Speed Profile",
             units="m/s",
-            comment="Speed average values from effective contour inwards to smallest contour, evenly spaced points",
+            comment="Speed averaged values from the effective contour inwards to the smallest contour, evenly spaced points",
         ),
     ),
     i=dict(
@@ -332,7 +348,7 @@ VAR_DESCR = dict(
         nc_attr=dict(
             long_name="Effective Radius",
             units="m",
-            comment="Radius of a circle whose area is equal to that enclosed by the effective contour",
+            comment="Radius of the best fit circle corresponding to the effective contour",
         ),
     ),
     radius_s=dict(
@@ -346,8 +362,7 @@ VAR_DESCR = dict(
         nc_attr=dict(
             long_name="Speed Radius",
             units="m",
-            comment="Radius of a circle whose area is equal to that "
-            "enclosed by the contour of maximum circum-average speed",
+            comment="Radius of the best fit circle corresponding to the contour of maximum circum-average speed",
         ),
     ),
     track=dict(
@@ -360,15 +375,55 @@ VAR_DESCR = dict(
             long_name="Trajectory number", comment="Trajectory identification number"
         ),
     ),
-    sub_track=dict(
+    segment=dict(
         attr_name=None,
-        nc_name="sub_track",
+        nc_name="segment",
         nc_type="uint32",
         nc_dims=("obs",),
         nc_attr=dict(
             long_name="Segment Number", comment="Segment number inside a group"
         ),
     ),
+    previous_obs=dict(
+        attr_name=None,
+        nc_name="previous_obs",
+        nc_type="int32",
+        nc_dims=("obs",),
+        nc_attr=dict(
+            long_name="Previous observation index",
+            comment="Index of previous observation in a spliting case",
+        ),
+    ),
+    next_obs=dict(
+        attr_name=None,
+        nc_name="next_obs",
+        nc_type="int32",
+        nc_dims=("obs",),
+        nc_attr=dict(
+            long_name="Next observation index",
+            comment="Index of next observation in a merging case",
+        ),
+    ),
+    previous_cost=dict(
+        attr_name=None,
+        nc_name="previous_cost",
+        nc_type="float32",
+        nc_dims=("obs",),
+        nc_attr=dict(
+            long_name="Previous cost for previous observation",
+            comment="",
+        ),
+    ),
+    next_cost=dict(
+        attr_name=None,
+        nc_name="next_cost",
+        nc_type="float32",
+        nc_dims=("obs",),
+        nc_attr=dict(
+            long_name="Next cost for next observation",
+            comment="",
+        ),
+    ),
     n=dict(
         attr_name=None,
         nc_name="observation_number",
@@ -419,9 +474,9 @@ VAR_DESCR = dict(
         nc_type="u2",
         nc_dims=("obs",),
         nc_attr=dict(
-            longname="number of point for effective contour",
+            long_name="number of points for effective contour",
             units="ordinal",
-            description="Number of point for effective contour, if greater than NbSample, there is a resampling",
+            description="Number of points for effective contour before resampling",
         ),
     ),
     contour_lon_s=dict(
@@ -463,9 +518,9 @@ VAR_DESCR = dict(
         nc_type="u2",
         nc_dims=("obs",),
         nc_attr=dict(
-            longname="number of point for speed contour",
+            long_name="number of points for speed contour",
             units="ordinal",
-            description="Number of point for speed contour, if greater than NbSample, there is a resampling",
+            description="Number of points for speed contour before resampling",
         ),
     ),
     shape_error_e=dict(
@@ -478,8 +533,8 @@ VAR_DESCR = dict(
         nc_dims=("obs",),
         nc_attr=dict(
             units="%",
-            comment="Error criterion between the effective contour and its fit with the circle of same effective radius",
-            long_name="Effective Contour Error",
+            comment="Error criterion between the effective contour and its best fit circle",
+            long_name="Effective Contour Shape Error",
         ),
     ),
     score=dict(
@@ -512,8 +567,8 @@ VAR_DESCR = dict(
         nc_dims=("obs",),
         nc_attr=dict(
             units="%",
-            comment="Error criterion between the speed contour and its fit with the circle of same speed radius",
-            long_name="Speed Contour Error",
+            comment="Error criterion between the speed contour and its best fit circle",
+            long_name="Speed Contour Shape Error",
         ),
     ),
     height_max_speed_contour=dict(
