@@ -173,6 +173,7 @@ class Amplitude(object):
                 self.mle,
                 -1,
             )
+            # After we use grid.data because index are in contour and we check before than no pixel are hide
             nb = len(lmi_i)
             if nb == 0:
                 logger.warning(
@@ -651,7 +652,13 @@ class Contours(object):
             if not overide_color:
                 ax.add_collection(LineCollection(paths, **local_kwargs))
         if display_criterion:
-            colors = {0: "g", 1: "r", 2: "b", 3: "k", 4: "y"}
+            colors = {
+                0: "limegreen",
+                1: "red",
+                2: "mediumblue",
+                3: "black",
+                4: "gold",
+            }
             for k, v in paths.items():
                 local_kwargs = kwargs.copy()
                 local_kwargs.pop("label", None)
