@@ -4,6 +4,7 @@ Base class to manage eddy observation
 """
 import logging
 from datetime import datetime
+from io import BufferedReader, BytesIO
 from tarfile import ExFileObject
 from tokenize import TokenError
 
@@ -32,6 +33,7 @@ from numpy import (
     isnan,
     linspace,
     ma,
+    nan,
     ndarray,
     ones,
     percentile,
@@ -63,8 +65,10 @@ from ..poly import (
     convexs,
     create_vertice,
     get_pixel_in_regular,
+    insidepoly,
+    poly_indexs,
+    reduce_size,
     vertice_overlap,
-    winding_number_poly,
 )
 
 logger = logging.getLogger("pet")
@@ -106,6 +110,28 @@ def shifted_ellipsoid_degrees_mask2(lon0, lat0, lon1, lat1, minor=1.5, major=1.5
     return m
 
 
+class Table(object):
+    def __init__(self, values):
+        self.values = values
+
+    def _repr_html_(self):
+        rows = list()
+        if isinstance(self.values, ndarray):
+            row = "\n".join([f"<td>{v}</td >" for v in self.values.dtype.names])
+            rows.append(f"<tr>{row}</tr>")
+        for row in self.values:
+            row = "\n".join([f"<td>{v}</td >" for v in row])
+            rows.append(f"<tr>{row}</tr>")
+        rows = "\n".join(rows)
+        return (
+            f'<font size="2">'
+            f'<table class="docutils align-default">'
+            f"{rows}"
+            f"</table>"
+            f"</font>"
+        )
+
+
 class EddiesObservations(object):
     """
     Class to store eddy observations.
@@ -142,6 +168,27 @@ class EddiesObservations(object):
         "height_inner_contour",
     ]
 
+    COLORS = [
+        "sienna",
+        "red",
+        "darkorange",
+        "gold",
+        "palegreen",
+        "limegreen",
+        "forestgreen",
+        "mediumblue",
+        "dodgerblue",
+        "lightskyblue",
+        "violet",
+        "blueviolet",
+        "darkmagenta",
+        "darkgrey",
+        "dimgrey",
+        "steelblue",
+    ]
+
+    NB_COLORS = len(COLORS)
+
     def __init__(
         self,
         size=0,
@@ -176,6 +223,10 @@ class EddiesObservations(object):
             return False
         return array_equal(self.obs, other.obs)
 
+    def get_color(self, i):
+        """Return colors like a cyclic list"""
+        return self.COLORS[i % self.NB_COLORS]
+
     @property
     def sign_legend(self):
         return "Cyclonic" if self.sign_type != 1 else "Anticyclonic"
@@ -200,11 +251,14 @@ class EddiesObservations(object):
         infos = self.get_infos()
         return f"""<b>{infos['nb_obs']} observations from {infos['t0']} to {infos['t1']} </b>"""
 
+    def parse_varname(self, name):
+        return self[name] if isinstance(name, str) else name
+
     def hist(self, varname, x, bins, percent=False, mean=False, nb=False):
         """Build histograms.
 
-        :param str varname: variable to use to compute stat
-        :param str x: variable to use to know in which bins
+        :param str,array varname: variable to use to compute stat
+        :param str,array x: variable to use to know in which bins
         :param array bins:
         :param bool percent: normalize by sum of all bins
         :param bool mean: compute mean by bins
@@ -212,14 +266,15 @@ class EddiesObservations(object):
         :return: value by bins
         :rtype: array
         """
+        x = self.parse_varname(x)
         if nb:
-            v = hist_numba(self[x], bins=bins)[0]
+            v = hist_numba(x, bins=bins)[0]
         else:
-            v = histogram(self[x], bins=bins, weights=self[varname])[0]
+            v = histogram(x, bins=bins, weights=self.parse_varname(varname))[0]
         if percent:
             v = v.astype("f4") / v.sum() * 100
         elif mean:
-            v /= hist_numba(self[x], bins=bins)[0]
+            v /= hist_numba(x, bins=bins)[0]
         return v
 
     @staticmethod
@@ -227,6 +282,25 @@ class EddiesObservations(object):
         """Return value evenly spaced with few numbers"""
         return "".join([f"{v_:10.2f}" for v_ in value])
 
+    def field_table(self):
+        """
+        Produce description table of the fields available in this object
+        """
+        rows = [("Name (Unit)", "Long name", "Scale factor", "Offset")]
+        names = list(self.obs.dtype.names)
+        names.sort()
+        for field in names:
+            infos = VAR_DESCR[field]
+            rows.append(
+                (
+                    f"{infos.get('nc_name', field)} ({infos['nc_attr'].get('units', '')})",
+                    infos["nc_attr"].get("long_name", "").capitalize(),
+                    infos.get("scale_factor", ""),
+                    infos.get("add_offset", ""),
+                )
+            )
+        return Table(rows)
+
     def __repr__(self):
         """
         Return general informations on dataset as strings.
@@ -310,6 +384,29 @@ class EddiesObservations(object):
             if candidate in handler.dimensions.keys():
                 return candidate
 
+    def remove_fields(self, *fields):
+        """
+        Copy with fields listed remove
+        """
+        nb_obs = self.obs.shape[0]
+        fields = set(fields)
+        only_variables = set(self.obs.dtype.names) - fields
+        track_extra_variables = set(self.track_extra_variables) - fields
+        array_variables = set(self.array_variables) - fields
+        new = self.__class__(
+            size=nb_obs,
+            track_extra_variables=track_extra_variables,
+            track_array_variables=self.track_array_variables,
+            array_variables=array_variables,
+            only_variables=only_variables,
+            raw_data=self.raw_data,
+        )
+        new.sign_type = self.sign_type
+        for name in new.obs.dtype.names:
+            logger.debug("Copy of field %s ...", name)
+            new.obs[name] = self.obs[name]
+        return new
+
     def add_fields(self, fields=list(), array_fields=list()):
         """
         Add a new field.
@@ -328,18 +425,17 @@ class EddiesObservations(object):
             raw_data=self.raw_data,
         )
         new.sign_type = self.sign_type
-        for field in self.obs.dtype.descr:
-            logger.debug("Copy of field %s ...", field)
-            var = field[0]
-            new.obs[var] = self.obs[var]
+        for name in self.obs.dtype.names:
+            logger.debug("Copy of field %s ...", name)
+            new.obs[name] = self.obs[name]
         return new
 
     def add_rotation_type(self):
         new = self.add_fields(("type_cyc",))
-        new.type_cyc = self.sign_type
+        new.type_cyc[:] = self.sign_type
         return new
 
-    def circle_contour(self, only_virtual=False):
+    def circle_contour(self, only_virtual=False, factor=1):
         """
         Set contours as a circles from radius and center data.
 
@@ -354,12 +450,12 @@ class EddiesObservations(object):
                 continue
             x, y = obs["lon"], obs["lat"]
             if radius_s:
-                r_s = obs["radius_s"]
+                r_s = obs["radius_s"] * factor
                 obs["contour_lon_s"], obs["contour_lat_s"] = local_to_coordinates(
                     x_norm * r_s, y_norm * r_s, x, y
                 )
             if radius_e:
-                r_e = obs["radius_e"]
+                r_e = obs["radius_e"] * factor
                 obs["contour_lon_e"], obs["contour_lat_e"] = local_to_coordinates(
                     x_norm * r_e, y_norm * r_e, x, y
                 )
@@ -450,24 +546,33 @@ class EddiesObservations(object):
         for obs in self.obs:
             yield obs
 
-    def iter_on(self, xname: str, bins=None):
+    def iter_on(self, xname, bins=None):
         """
         Yield observation group for each bin.
 
-        :param str xname:
+        :param str,array xname:
         :param array bins: bounds of each bin ,
-        :return: Group observations
-        :rtype: self.__class__
+        :return: index or mask, bound low, bound up
         """
-        x = self[xname]
+        x = self[xname] if isinstance(xname, str) else xname
         d = x[1:] - x[:-1]
         if bins is None:
             bins = arange(x.min(), x.max() + 2)
+        elif not isinstance(bins, ndarray):
+            bins = array(bins)
         nb_bins = len(bins) - 1
-        i = digitize(x, bins) - 1
+
         # Not monotonous
         if (d < 0).any():
+            # If bins cover a small part of value
+            test, translate, x = iter_mode_reduce(x, bins)
+            # convert value in bins number
+            i = numba_digitize(x, bins) - 1
+            # Order by bins
             i_sort = i.argsort()
+            # If in reduce mode we will translate i_sort in full array index
+            i_sort_ = translate[i_sort] if test else i_sort
+            # Bound for each bins in sorting view
             i0, i1, _ = build_index(i[i_sort])
             m = ~(i0 == i1)
             i0, i1 = i0[m], i1[m]
@@ -475,8 +580,9 @@ class EddiesObservations(object):
                 i_bins = i[i_sort[i0_]]
                 if i_bins == -1 or i_bins == nb_bins:
                     continue
-                yield i_sort[i0_:i1_], bins[i_bins], bins[i_bins + 1]
+                yield i_sort_[i0_:i1_], bins[i_bins], bins[i_bins + 1]
         else:
+            i = numba_digitize(x, bins) - 1
             i0, i1, _ = build_index(i)
             m = ~(i0 == i1)
             i0, i1 = i0[m], i1[m]
@@ -488,10 +594,8 @@ class EddiesObservations(object):
         """
         Align the time indexes of two datasets.
         """
-        iter_self, iter_other = (
-            self.iter_on(var_name, **kwargs),
-            other.iter_on(var_name, **kwargs),
-        )
+        iter_self = self.iter_on(var_name, **kwargs)
+        iter_other = other.iter_on(var_name, **kwargs)
         indexs_other, b0_other, b1_other = iter_other.__next__()
         for indexs_self, b0_self, b1_self in iter_self:
             if b0_self > b0_other:
@@ -547,9 +651,9 @@ class EddiesObservations(object):
     def copy(self):
         return self.__copy__()
 
-    @staticmethod
-    def new_like(eddies, new_size: int):
-        return eddies.__class__(
+    @classmethod
+    def new_like(cls, eddies, new_size: int):
+        return cls(
             new_size,
             track_extra_variables=eddies.track_extra_variables,
             track_array_variables=eddies.track_array_variables,
@@ -610,8 +714,12 @@ class EddiesObservations(object):
         )
         if isinstance(filename, zarr.storage.MutableMapping):
             return cls.load_from_zarr(filename, **kwargs)
-        end = b".zarr" if isinstance(filename_, bytes) else ".zarr"
-        if filename_.endswith(end):
+        if isinstance(filename, (bytes, str)):
+            end = b".zarr" if isinstance(filename_, bytes) else ".zarr"
+            zarr_file = filename_.endswith(end)
+        else:
+            zarr_file = False
+        if zarr_file:
             return cls.load_from_zarr(filename, **kwargs)
         else:
             return cls.load_from_netcdf(filename, **kwargs)
@@ -639,8 +747,8 @@ class EddiesObservations(object):
         :return: Obsevations selected
         :return type: class
         """
-        # FIXME must be investigate, in zarr no dimensions name (or could be add in attr)
-        array_dim = 50
+        # FIXME
+        array_dim = -1
         if isinstance(filename, zarr.storage.MutableMapping):
             h_zarr = filename
         else:
@@ -650,6 +758,10 @@ class EddiesObservations(object):
         var_list = cls.build_var_list(list(h_zarr.keys()), remove_vars, include_vars)
 
         nb_obs = getattr(h_zarr, var_list[0]).shape[0]
+        dims = list(cls.zarr_dimension(filename))
+        if len(dims) == 2 and nb_obs in dims:
+            # FIXME must be investigate, in zarr no dimensions name (or could be add in attr)
+            array_dim = dims[1] if nb_obs == dims[0] else dims[0]
         if indexs is not None and "obs" in indexs:
             sl = indexs["obs"]
             sl = slice(sl.start, min(sl.stop, nb_obs))
@@ -662,7 +774,7 @@ class EddiesObservations(object):
                 logger.warning("step of slice won't be use")
         logger.debug("%d observations will be load", nb_obs)
         kwargs = dict()
-        dims = cls.zarr_dimension(filename)
+
         if array_dim in dims:
             kwargs["track_array_variables"] = array_dim
             kwargs["array_variables"] = list()
@@ -729,10 +841,10 @@ class EddiesObservations(object):
         :param zarr_dataset handler_zarr:
         :param array handler_eddies:
         :param slice zarr_dataset sl_obs:
-        :param int zarr_dataset buffer_size:
-        :param float zarr_dataset factor:
-        :param bool zarr_dataset raw_data:
-        :param None,float zarr_dataset scale_factor:
+        :param int buffer_size:
+        :param float factor:
+        :param bool raw_data:
+        :param None,float scale_factor:
         :param None,float add_offset:
         """
         i_start, i_stop = sl_obs.start, sl_obs.stop
@@ -777,7 +889,7 @@ class EddiesObservations(object):
         array_dim = "NbSample"
         if isinstance(filename, bytes):
             filename = filename.astype(str)
-        if isinstance(filename, ExFileObject):
+        if isinstance(filename, (ExFileObject, BufferedReader, BytesIO)):
             filename.seek(0)
             args, kwargs = ("in-mem-file",), dict(memory=filename.read())
         else:
@@ -996,10 +1108,23 @@ class EddiesObservations(object):
             labels = [VAR_DESCR[label]["nc_name"] for label in labels]
         return labels
 
-    def match(self, other, method="overlap", intern=False, cmin=0, **kwargs):
+    def match(
+        self,
+        other,
+        i_self=None,
+        i_other=None,
+        method="overlap",
+        intern=False,
+        cmin=0,
+        **kwargs,
+    ):
         """Return index and score computed on the effective contour.
 
         :param EddiesObservations other: Observations to compare
+        :param array[bool,int],None i_self:
+            Index or mask to subset observations, it could avoid to build a specific dataset.
+        :param array[bool,int],None i_other:
+            Index or mask to subset observations, it could avoid to build a specific dataset.
         :param str method:
             - "overlap": the score is computed with contours;
             - "circle": circles are computed and used for score (TODO)
@@ -1012,28 +1137,38 @@ class EddiesObservations(object):
 
         .. minigallery:: py_eddy_tracker.EddiesObservations.match
         """
-        # if method is "overlap" method will use contour to compute score,
-        # if method is "circle" method will apply a formula of circle overlap
         x_name, y_name = self.intern(intern)
+        if i_self is None:
+            i_self = slice(None)
+        if i_other is None:
+            i_other = slice(None)
         if method == "overlap":
-            i, j = bbox_intersection(
-                self[x_name], self[y_name], other[x_name], other[y_name]
-            )
-            c = vertice_overlap(
-                self[x_name][i],
-                self[y_name][i],
-                other[x_name][j],
-                other[y_name][j],
-                **kwargs,
-            )
+            x0, y0 = self[x_name][i_self], self[y_name][i_self]
+            x1, y1 = other[x_name][i_other], other[y_name][i_other]
+            i, j = bbox_intersection(x0, y0, x1, y1)
+            c = vertice_overlap(x0[i], y0[i], x1[j], y1[j], **kwargs)
         elif method == "close_center":
-            i, j, c = close_center(
-                self.latitude, self.longitude, other.latitude, other.longitude, **kwargs
-            )
-
+            x0, y0 = self.longitude[i_self], self.latitude[i_self]
+            x1, y1 = other.longitude[i_other], other.latitude[i_other]
+            i, j, c = close_center(x0, y0, x1, y1, **kwargs)
         m = c >= cmin  # ajout >= pour garder la cmin dans la sélection
         return i[m], j[m], c[m]
 
+    @staticmethod
+    def re_reference_index(index, ref):
+        """
+        Shift index with ref
+
+        :param array,int index: local index to re ref
+        :param slice,array ref:
+            reference could be a slice in this case we juste add start to index
+            or could be indexs and in this case we need to translate
+        """
+        if isinstance(ref, slice):
+            return index + ref.start
+        else:
+            return ref[index]
+
     @classmethod
     def cost_function_common_area(cls, xy_in, xy_out, distance, intern=False):
         """How does it work on x bound ?
@@ -1192,8 +1327,7 @@ class EddiesObservations(object):
         """Write something (TODO)"""
         mask = ~cost.mask
         # Count number of link by self obs and other obs
-        self_links = mask.sum(axis=1)
-        other_links = mask.sum(axis=0)
+        self_links, other_links = sum_row_column(mask)
         max_links = max(self_links.max(), other_links.max())
         if max_links > 5:
             logger.warning("One observation have %d links", max_links)
@@ -1294,7 +1428,7 @@ class EddiesObservations(object):
         return mask
 
     def solve_function(self, cost_matrix):
-        return where(self.solve_simultaneous(cost_matrix))
+        return numba_where(self.solve_simultaneous(cost_matrix))
 
     def post_process_link(self, other, i_self, i_other):
         if unique(i_other).shape[0] != i_other.shape[0]:
@@ -1533,7 +1667,7 @@ class EddiesObservations(object):
             filename = filename.replace(".nc", ".zarr")
         if filename.endswith(".zarr"):
             zarr_flag = True
-        logger.info("Store in %s", filename)
+        logger.info("Store in %s (%d observations)", filename, len(self))
         if zarr_flag:
             handler = zarr.open(filename, "w")
             self.to_zarr(handler, **kwargs)
@@ -1560,6 +1694,26 @@ class EddiesObservations(object):
         for key, item in self.global_attr.items():
             h_nc.setncattr(key, item)
 
+    def mask_from_polygons(self, polygons):
+        """
+        Return mask for all observation in one of polygons list
+
+        :param list((array,array)) polygons: list of x/y array which be used to identify observations
+        """
+        x, y = polygons[0]
+        m = insidepoly(
+            self.longitude, self.latitude, x.reshape((1, -1)), y.reshape((1, -1))
+        )
+        for x, y in polygons[1:]:
+            m_ = ~m
+            m[m_] = insidepoly(
+                self.longitude[m_],
+                self.latitude[m_],
+                x.reshape((1, -1)),
+                y.reshape((1, -1)),
+            )
+        return m
+
     def extract_with_area(self, area, **kwargs):
         """
         Extract geographically with a bounding box.
@@ -1575,12 +1729,24 @@ class EddiesObservations(object):
 
         .. minigallery:: py_eddy_tracker.EddiesObservations.extract_with_area
         """
-        mask = (self.latitude > area["llcrnrlat"]) * (self.latitude < area["urcrnrlat"])
+        lat0 = area.get("llcrnrlat", -90)
+        lat1 = area.get("urcrnrlat", 90)
+        mask = (self.latitude > lat0) * (self.latitude < lat1)
         lon0 = area["llcrnrlon"]
         lon = (self.longitude - lon0) % 360 + lon0
         mask *= (lon > lon0) * (lon < area["urcrnrlon"])
         return self.extract_with_mask(mask, **kwargs)
 
+    def time_sub_sample(self, t0, time_step):
+        """
+        Time sub sampling
+
+        :param int,float t0: reference time which will be keep
+        :param int,float time_step: keep every observation spaced by time_step
+        """
+        mask = (self.time - t0) % time_step == 0
+        return self.extract_with_mask(mask)
+
     def extract_with_mask(self, mask):
         """
         Extract a subset of observations.
@@ -1589,11 +1755,6 @@ class EddiesObservations(object):
         :return: same object with selected observations
         :rtype: self
         """
-        # ça n'existe plus ça??
-        # full_path=False,
-        # remove_incomplete=False,
-        # compress_id=False,
-        # reject_virtual=False
 
         nb_obs = mask.sum()
         new = self.__class__.new_like(self, nb_obs)
@@ -1626,7 +1787,7 @@ class EddiesObservations(object):
             x = (x - ref) % 360 + ref
         kwargs = kwargs.copy()
         if name is not None and "c" not in kwargs:
-            v = self[name] if isinstance(name, str) else name
+            v = self.parse_varname(name)
             kwargs["c"] = v * factor
         return ax.scatter(x, self.latitude, **kwargs)
 
@@ -1671,7 +1832,7 @@ class EddiesObservations(object):
         if "facecolors" not in kwargs:
             kwargs = kwargs.copy()
             cmap = get_cmap(cmap, lut)
-            v = (self[varname] if isinstance(varname, str) else varname) * factor
+            v = self.parse_varname(varname) * factor
             if vmin is None:
                 vmin = v.min()
             if vmax is None:
@@ -1679,6 +1840,8 @@ class EddiesObservations(object):
             v = (v - vmin) / (vmax - vmin)
             colors = [cmap(v_) for v_ in v]
             kwargs["facecolors"] = colors
+        if "label" in kwargs:
+            kwargs["label"] = self.format_label(kwargs["label"])
         c = PolyCollection(verts, **kwargs)
         ax.add_collection(c)
         c.cmap = cmap
@@ -1783,7 +1946,7 @@ class EddiesObservations(object):
 
         .. minigallery:: py_eddy_tracker.EddiesObservations.bins_stat
         """
-        v = self[xname] if isinstance(xname, str) else xname
+        v = self.parse_varname(xname)
         mask = self.merge_filters(mask)
         v = v[mask]
         if bins is None:
@@ -1791,7 +1954,7 @@ class EddiesObservations(object):
         y, x = hist_numba(v, bins=bins)
         x = (x[1:] + x[:-1]) / 2
         if method == "mean":
-            y_v = self[yname] if isinstance(yname, str) else yname
+            y_v = self.parse_varname(yname)
             y_v = y_v[mask]
             y_, _ = histogram(v, bins=bins, weights=y_v)
             with errstate(divide="ignore", invalid="ignore"):
@@ -1876,6 +2039,19 @@ class EddiesObservations(object):
         xname, yname = self.intern(intern)
         return convexs(self[xname], self[yname])
 
+    def contains(self, x, y, intern=False):
+        """
+        Return index of contour which contain (x,y)
+
+        :param array x: longitude
+        :param array y: latitude
+        :param bool intern: If true use speed contour instead of effective contour
+        :return: indexs, -1 if no index
+        :rtype: array[int32]
+        """
+        xname, yname = self.intern(intern)
+        return poly_indexs(x, y, self[xname], self[yname])
+
     def inside(self, x, y, intern=False):
         """
         True for each point inside the effective contour of an eddy
@@ -1887,6 +2063,7 @@ class EddiesObservations(object):
         :rtype: array[bool]
         """
         xname, yname = self.intern(intern)
+        # FIXME: wrapping
         return insidepoly(x, y, self[xname], self[yname])
 
     def grid_count(self, bins, intern=False, center=False, filter=slice(None)):
@@ -1948,10 +2125,10 @@ class EddiesObservations(object):
 
     def grid_box_stat(self, bins, varname, method=50, data=None, filter=slice(None)):
         """
-        Compute mean of eddies in each bin
+        Get percentile of eddies in each bin
 
         :param (numpy.array,numpy.array) bins: bins (grid) to count
-        :param str varname: variable to apply the method
+        :param str varname: variable to apply the method if data is None and will be output name
         :param str,float method: method to apply. If float, use ?
         :param array data: Array used to compute stat if defined
         :param array,mask,slice filter: keep the data selected with the filter
@@ -1999,7 +2176,7 @@ class EddiesObservations(object):
         Return the mean of the eddies' variable in each bin
 
         :param (numpy.array,numpy.array) bins: bins (grid) to compute the mean on
-        :param str varname: name of variable to compute the mean on
+        :param str varname: name of variable to compute the mean on and output grid_name
         :param array data: Array used to compute stat if defined
         :return: return the gridde mean variable
         :rtype: py_eddy_tracker.dataset.grid.RegularGridDataset
@@ -2071,13 +2248,16 @@ class EddiesObservations(object):
     @property
     def period(self):
         """
-        Give the time coverage
+        Give the time coverage. If collection is empty, return nan,nan
 
         :return: first and last date
         :rtype: (int,int)
         """
         if self.period_ is None:
-            self.period_ = self.time.min(), self.time.max()
+            if self.time.size < 1:
+                self.period_ = nan, nan
+            else:
+                self.period_ = self.time.min(), self.time.max()
         return self.period_
 
     @property
@@ -2136,6 +2316,7 @@ def grid_count_pixel_in(
     for i_ in range(nb):
         x_, y_, x_ref_ = x[i_], y[i_], x_ref[i_]
         x_ = (x_ - x_ref_) % 360 + x_ref_
+        x_, y_ = reduce_size(x_, y_)
         v = create_vertice(x_, y_)
         (x_start, x_stop), (y_start, y_stop) = bbox_indice_regular(
             v,
@@ -2147,38 +2328,10 @@ def grid_count_pixel_in(
             is_circular,
             x_size,
         )
-
         i, j = get_pixel_in_regular(v, x_c, y_c, x_start, x_stop, y_start, y_stop)
         grid_count_(grid, i, j)
 
 
-@njit(cache=True)
-def insidepoly(x_p, y_p, x_c, y_c):
-    """
-    True for each postion inside a contour
-
-    :param array x_p: longitude to test
-    :param array y_p: latitude to test
-    :param array x_c: longitude of contours
-    :param array y_c: latitude of contours
-    """
-    nb_p = x_p.shape[0]
-    nb_c = x_c.shape[0]
-    flag = zeros(nb_p, dtype=numba_types.bool_)
-    for i in range(nb_c):
-        x_c_min, y_c_min = x_c[i].min(), y_c[i].min()
-        x_c_max, y_c_max = x_c[i].max(), y_c[i].max()
-        v = create_vertice(x_c[i], y_c[i])
-        for j in range(nb_p):
-            if flag[j]:
-                continue
-            x, y = x_p[j], y_p[j]
-            if x > x_c_min and x < x_c_max and y > y_c_min and y < y_c_max:
-                if winding_number_poly(x, y, v) != 0:
-                    flag[j] = True
-    return flag
-
-
 @njit(cache=True)
 def grid_box_stat(x_c, y_c, grid, mask, x, y, value, circular=False, method=50):
     """
@@ -2223,7 +2376,6 @@ def grid_box_stat(x_c, y_c, grid, mask, x, y, value, circular=False, method=50):
         if i_ != i0 or j_ != j0:
             # apply method and store result
             grid[i_, j_] = percentile(values, method)
-            # grid[i_, j_] = len(values)
             mask[i_, j_] = False
             # start new group
             i0, j0 = i_, j_
@@ -2246,12 +2398,14 @@ def grid_stat(x_c, y_c, grid, x, y, result, circular=False, method="mean"):
     :param bool circular: True if grid is wrappable
     :param str method: 'mean', 'max'
     """
+    # FIXME : how does it work on grid bound
     nb = result.shape[0]
     xstep, ystep = x_c[1] - x_c[0], y_c[1] - y_c[0]
     x0, y0 = x_c - xstep / 2.0, y_c - ystep / 2.0
     nb_x = x_c.shape[0]
     max_method = "max" == method
     mean_method = "mean" == method
+    count_method = "count" == method
     for elt in range(nb):
         v = create_vertice(x[elt], y[elt])
         (x_start, x_stop), (y_start, y_stop) = bbox_indice_regular(
@@ -2259,11 +2413,18 @@ def grid_stat(x_c, y_c, grid, x, y, result, circular=False, method="mean"):
         )
         i, j = get_pixel_in_regular(v, x_c, y_c, x_start, x_stop, y_start, y_stop)
 
-        if mean_method:
+        if count_method:
+            result[elt] = i.shape[0]
+        elif mean_method:
             v_sum = 0
             for i_, j_ in zip(i, j):
                 v_sum += grid[i_, j_]
-            result[elt] = v_sum / i.shape[0]
+            nb_ = i.shape[0]
+            # FIXME : how does it work on grid bound,
+            if nb_ == 0:
+                result[elt] = nan
+            else:
+                result[elt] = v_sum / nb_
         elif max_method:
             v_max = -1e40
             for i_, j_ in zip(i, j):
@@ -2281,3 +2442,90 @@ class VirtualEddiesObservations(EddiesObservations):
         elements = super().elements
         elements.extend(["track", "segment_size", "dlon", "dlat"])
         return list(set(elements))
+
+
+@njit(cache=True)
+def numba_where(mask):
+    """Usefull when mask is close to be empty"""
+    return where(mask)
+
+
+@njit(cache=True)
+def sum_row_column(mask):
+    """
+    Compute sum on row and column at same time
+    """
+    nb_x, nb_y = mask.shape
+    row_sum = zeros(nb_x, dtype=numba_types.int32)
+    column_sum = zeros(nb_y, dtype=numba_types.int32)
+    for i in range(nb_x):
+        for j in range(nb_y):
+            if mask[i, j]:
+                row_sum[i] += 1
+                column_sum[j] += 1
+    return row_sum, column_sum
+
+
+@njit(cache=True)
+def numba_digitize(values, bins):
+    # Check if bins are regular
+    nb_bins = bins.shape[0]
+    step = bins[1] - bins[0]
+    bin_previous = bins[1]
+    for i in range(2, nb_bins):
+        bin_current = bins[i]
+        if step != (bin_current - bin_previous):
+            # If bins are not regular
+            return digitize(values, bins)
+        bin_previous = bin_current
+    nb_values = values.shape[0]
+    out = empty(nb_values, dtype=numba_types.int64)
+    up, down = bins[0], bins[-1]
+    for i in range(nb_values):
+        v_ = values[i]
+        if v_ >= down:
+            out[i] = nb_bins
+            continue
+        if v_ < up:
+            out[i] = 0
+            continue
+        out[i] = (v_ - bins[0]) / step + 1
+    return out
+
+
+@njit(cache=True)
+def iter_mode_reduce(x, bins):
+    """
+    Test if we could use a reduce mode
+
+    :param array x: array to divide in group
+    :param array bins: array which defined bounds between each group
+    :return: If reduce mode, translator, and reduce x
+    """
+    nb = x.shape[0]
+    # If we use less than half value
+    limit = nb // 2
+    # low and up
+    x0, x1 = bins[0], bins[-1]
+    m = empty(nb, dtype=numba_types.bool_)
+    # To count number of value cover by bins
+    c = 0
+    for i in range(nb):
+        x_ = x[i]
+        test = (x_ >= x0) * (x_ <= x1)
+        m[i] = test
+        if test:
+            c += 1
+            # If number value exceed limit
+            if c > limit:
+                return False, empty(0, dtype=numba_types.int_), x
+    # Indices to be able to translate in full index array
+    indices = empty(c, dtype=numba_types.int_)
+    x_ = empty(c, dtype=x.dtype)
+    j = 0
+    for i in range(nb):
+        if m[i]:
+            indices[j] = i
+            x_[j] = x[i]
+            j += 1
+    return True, indices, x_
