@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-Class to manage observations gathered in track
+Class to manage observations gathered in trajectories
 """
 import logging
 from datetime import datetime, timedelta
@@ -16,27 +16,24 @@ from numpy import (
     degrees,
     empty,
     histogram,
-    interp,
     median,
     nan,
     ones,
     radians,
     sin,
     unique,
-    where,
     zeros,
 )
-from Polygon import Polygon
 
 from .. import VAR_DESCR_inv, __version__
 from ..generic import build_index, cumsum_by_track, distance, split_line, wrap_longitude
-from ..poly import create_vertice_from_2darray, merge, polygon_overlap
-from .observation import EddiesObservations
+from ..poly import bbox_intersection, merge, vertice_overlap
+from .groups import GroupEddiesObservations, get_missing_indices
 
 logger = logging.getLogger("pet")
 
 
-class TrackEddiesObservations(EddiesObservations):
+class TrackEddiesObservations(GroupEddiesObservations):
     """Class to practice Tracking on observations"""
 
     __slots__ = ("__obs_by_track", "__first_index_of_track", "__nb_track")
@@ -79,6 +76,26 @@ class TrackEddiesObservations(EddiesObservations):
                 continue
             yield self.index(slice(i0, i0 + nb))
 
+    def get_missing_indices(self, dt):
+        """find indices where observations is missing.
+
+        :param int,float dt: theorical delta time between 2 observations
+        """
+        return get_missing_indices(
+            self.time,
+            self.track,
+            dt=dt,
+            flag_untrack=False,
+            indice_untrack=self.NOGROUP,
+        )
+
+    def fix_next_previous_obs(self):
+        """function used after 'insert_virtual', to correct next_obs and
+        previous obs.
+        """
+
+        pass
+
     @property
     def nb_tracks(self):
         """
@@ -124,7 +141,7 @@ class TrackEddiesObservations(EddiesObservations):
         return content
 
     def add_distance(self):
-        """Add a field of distance (m) between to consecutive observation, 0 for the last observation of each track"""
+        """Add a field of distance (m) between two consecutive observations, 0 for the last observation of each track"""
         if "distance_next" in self.observations.dtype.descr:
             return self
         new = self.add_fields(("distance_next",))
@@ -148,41 +165,34 @@ class TrackEddiesObservations(EddiesObservations):
         d_[-1] = 0
         return d_
 
-    def filled_by_interpolation(self, mask):
-        """Filled selected values by interpolation
+    def normalize_longitude(self):
+        """Normalize all longitude
 
-        :param array(bool) mask: True if must be filled by interpolation
-
-        .. minigallery:: py_eddy_tracker.TrackEddiesObservations.filled_by_interpolation
+        Normalize longitude field and in the same range :
+        - longitude_max
+        - contour_lon_e (how to do if in raw)
+        - contour_lon_s (how to do if in raw)
         """
-        nb_filled = mask.sum()
-        logger.info("%d obs will be filled (unobserved)", nb_filled)
-
-        nb_obs = len(self)
-        index = arange(nb_obs)
-
-        for field in self.obs.dtype.descr:
-            var = field[0]
-            if (
-                var in ["n", "virtual", "track", "cost_association"]
-                or var in self.array_variables
-            ):
-                continue
-            # to normalize longitude before interpolation
-            if var == "lon":
-                lon = self.lon
-                first = where(self.n == 0)[0]
-                nb_obs = empty(first.shape, dtype="u4")
-                nb_obs[:-1] = first[1:] - first[:-1]
-                nb_obs[-1] = lon.shape[0] - first[-1]
-                lon0 = (lon[first] - 180).repeat(nb_obs)
-                self.lon[:] = (lon - lon0) % 360 + lon0
-            self.obs[var][mask] = interp(
-                index[mask], index[~mask], self.obs[var][~mask]
-            )
+        lon0 = (self.lon[self.index_from_track] - 180).repeat(self.nb_obs_by_track)
+        logger.debug("Normalize longitude")
+        self.lon[:] = (self.lon - lon0) % 360 + lon0
+        if "lon_max" in self.obs.dtype.names:
+            logger.debug("Normalize longitude_max")
+            self.lon_max[:] = (self.lon_max - self.lon + 180) % 360 + self.lon - 180
+        if not self.raw_data:
+            if "contour_lon_e" in self.obs.dtype.names:
+                logger.debug("Normalize effective contour longitude")
+                self.contour_lon_e[:] = (
+                    (self.contour_lon_e.T - self.lon + 180) % 360 + self.lon - 180
+                ).T
+            if "contour_lon_s" in self.obs.dtype.names:
+                logger.debug("Normalize speed contour longitude")
+                self.contour_lon_s[:] = (
+                    (self.contour_lon_s.T - self.lon + 180) % 360 + self.lon - 180
+                ).T
 
     def extract_longer_eddies(self, nb_min, nb_obs, compress_id=True):
-        """Select eddies which are longer than nb_min"""
+        """Select the trajectories longer than nb_min"""
         mask = nb_obs >= nb_min
         nb_obs_select = mask.sum()
         logger.info("Selection of %d observations", nb_obs_select)
@@ -227,9 +237,9 @@ class TrackEddiesObservations(EddiesObservations):
 
     def extract_with_period(self, period, **kwargs):
         """
-        Extract with a period
+        Extract within a time period
 
-        :param (int,int) period: two date to define period, must be specify from 1/1/1950
+        :param (int,int) period: two dates to define the period, must be specify from 1/1/1950
         :param dict kwargs: look at :py:meth:`extract_with_mask`
         :return: Return all eddy tracks which are in bounds
         :rtype: TrackEddiesObservations
@@ -252,9 +262,9 @@ class TrackEddiesObservations(EddiesObservations):
 
     def get_azimuth(self, equatorward=False):
         """
-        Return azimuth for each tracks.
+        Return azimuth for each track.
 
-        Azimuth is compute with first and last observation
+        Azimuth is computed with first and last observation
 
         :param bool equatorward: If True,  Poleward are positive and equatorward negative
         :rtype: array
@@ -286,7 +296,7 @@ class TrackEddiesObservations(EddiesObservations):
         """
         if self.__first_index_of_track is None:
             s = self.tracks.max() + 1
-            # Doesn't work => core dump with numba, maybe he wait i8 instead of u4
+            # Doesn't work => core dump with numba, maybe he wants i8 instead of u4
             # self.__first_index_of_track = -ones(s, self.tracks.dtype)
             # self.__obs_by_track = zeros(s, self.observation_number.dtype)
             self.__first_index_of_track = -ones(s, "i8")
@@ -334,12 +344,12 @@ class TrackEddiesObservations(EddiesObservations):
 
     @property
     def lifetime(self):
-        """Return for each observation lifetime"""
+        """Return lifetime for each observation"""
         return self.nb_obs_by_track.repeat(self.nb_obs_by_track)
 
     @property
     def age(self):
-        """Return for each observation age in %, will be [0:100]"""
+        """Return age in % for each observation, will be [0:100]"""
         return self.n.astype("f4") / (self.lifetime - 1) * 100.0
 
     def extract_ids(self, tracks):
@@ -348,10 +358,10 @@ class TrackEddiesObservations(EddiesObservations):
 
     def extract_toward_direction(self, west=True, delta_lon=None):
         """
-        Get eddy which go in same direction
+        Get trajectories going in the same direction
 
-        :param bool west: Only eastward eddy if True return westward
-        :param None,float delta_lon: Only eddy with more than delta_lon span in longitude
+        :param bool west: Only eastward eddies if True return westward
+        :param None,float delta_lon: Only eddies with more than delta_lon span in longitude
         :return: Only eastern eddy
         :rtype: __class__
 
@@ -398,10 +408,10 @@ class TrackEddiesObservations(EddiesObservations):
 
     def extract_with_length(self, bounds):
         """
-        Return all observations in [b0:b1]
+        Return the observations within trajectories lasting between [b0:b1]
 
-        :param (int,int) bounds: length min and max of selected eddies, if use of -1 this bound is not used
-        :return: Return all eddy tracks which have length between bounds
+        :param (int,int) bounds: length min and max of the desired trajectories, if -1 this bound is not used
+        :return: Return all trajectories having length between bounds
         :rtype: TrackEddiesObservations
 
         .. minigallery:: py_eddy_tracker.TrackEddiesObservations.extract_with_length
@@ -433,13 +443,13 @@ class TrackEddiesObservations(EddiesObservations):
             return self
 
     def median_filter(self, half_window, xfield, yfield, inplace=True):
-        track = self.track
-        x = self.obs[xfield]
-        y = self.obs[yfield]
-        result = track_median_filter(half_window, x, y, track)
+        result = track_median_filter(
+            half_window, self[xfield], self[yfield], self.track
+        )
         if inplace:
-            self.obs[yfield] = result
+            self[yfield][:] = result
             return self
+        return result
 
     def position_filter(self, median_half_window, loess_half_window):
         self.median_filter(median_half_window, "time", "lon").loess_filter(
@@ -461,11 +471,11 @@ class TrackEddiesObservations(EddiesObservations):
         Extract a subset of observations
 
         :param array(bool) mask: mask to select observations
-        :param bool full_path: extract full path if only one part is selected
-        :param bool remove_incomplete: delete path which are not fully selected
-        :param bool compress_id: resample track number to use a little range
-        :param bool reject_virtual: if track are only virtual in selection we remove track
-        :return: same object with selected observations
+        :param bool full_path: extract the full trajectory if only one part is selected
+        :param bool remove_incomplete: delete trajectory if not fully selected
+        :param bool compress_id: resample trajectory number to use a smaller range
+        :param bool reject_virtual: if only virtual are selected, the trajectory is removed
+        :return: same object with the selected observations
         :rtype: self.__class__
         """
         if full_path and remove_incomplete:
@@ -501,16 +511,11 @@ class TrackEddiesObservations(EddiesObservations):
                 new.track = id_translate[new.track]
         return new
 
-    @staticmethod
-    def re_reference_index(index, ref):
-        if isinstance(ref, slice):
-            return index + ref.start
-        else:
-            return ref[index]
-
     def shape_polygon(self, intern=False):
         """
-        Get polygons which enclosed each track
+        Get the polygon enclosing each trajectory.
+
+        The polygon merges the non-overlapping bounds of the specified contours
 
         :param bool intern: If True use speed contour instead of effective contour
         :rtype: list(array, array)
@@ -520,9 +525,9 @@ class TrackEddiesObservations(EddiesObservations):
 
     def display_shape(self, ax, ref=None, intern=False, **kwargs):
         """
-        This function will draw shape of each track
+        This function will draw the shape of each trajectory
 
-        :param matplotlib.axes.Axes ax: ax where drawed
+        :param matplotlib.axes.Axes ax: ax to draw
         :param float,int ref: if defined all coordinates will be wrapped with ref like west boundary
         :param bool intern: If True use speed contour instead of effective contour
         :param dict kwargs: keyword arguments for Axes.plot
@@ -547,10 +552,10 @@ class TrackEddiesObservations(EddiesObservations):
 
     def close_tracks(self, other, nb_obs_min=10, **kwargs):
         """
-        Get close from another atlas.
+        Get close trajectories from another atlas.
 
         :param self other: Atlas to compare
-        :param int nb_obs_min: Minimal number of overlap for one track
+        :param int nb_obs_min: Minimal number of overlap for one trajectory
         :param dict kwargs: keyword arguments for match function
         :return: return other atlas reduce to common track with self
 
@@ -560,7 +565,7 @@ class TrackEddiesObservations(EddiesObservations):
         p0, p1 = self.period
         indexs = list()
         for i_self, i_other, t0, t1 in self.align_on(other, bins=range(p0, p1 + 2)):
-            i, j, s = self.index(i_self).match(other.index(i_other), **kwargs)
+            i, j, s = self.match(other, i_self=i_self, i_other=i_other, **kwargs)
             indexs.append(other.re_reference_index(j, i_other))
         indexs = concatenate(indexs)
         tr, nb = unique(other.track[indexs], return_counts=True)
@@ -577,10 +582,10 @@ class TrackEddiesObservations(EddiesObservations):
 
     def plot(self, ax, ref=None, **kwargs):
         """
-        This function will draw path of each track
+        This function will draw path of each trajectory
 
-        :param matplotlib.axes.Axes ax: ax where drawed
-        :param float,int ref: if defined all coordinates will be wrapped with ref like west boundary
+        :param matplotlib.axes.Axes ax: ax to draw
+        :param float,int ref: if defined, all coordinates will be wrapped with ref like west boundary
         :param dict kwargs: keyword arguments for Axes.plot
         :return: matplotlib mappable
         """
@@ -595,14 +600,14 @@ class TrackEddiesObservations(EddiesObservations):
         return ax.plot(x, y, **kwargs)
 
     def split_network(self, intern=True, **kwargs):
-        """Divide each group in track"""
+        """Return each group (network) divided in segments"""
         track_s, track_e, track_ref = build_index(self.tracks)
         ids = empty(
             len(self),
             dtype=[
                 ("group", self.tracks.dtype),
                 ("time", self.time.dtype),
-                ("track", "u2"),
+                ("track", "u4"),
                 ("previous_cost", "f4"),
                 ("next_cost", "f4"),
                 ("previous_obs", "i4"),
@@ -610,47 +615,37 @@ class TrackEddiesObservations(EddiesObservations):
             ],
         )
         ids["group"], ids["time"] = self.tracks, self.time
-        # To store id track
+        # Initialisation
+        # To store the id of the segments, the backward and forward cost associations
         ids["track"], ids["previous_cost"], ids["next_cost"] = 0, 0, 0
+        # To store the indexes of the backward and forward observations associated
         ids["previous_obs"], ids["next_obs"] = -1, -1
+        # At the end, ids["previous_obs"] == -1 means the start of a non-split segment
+        # and ids["next_obs"] == -1 means the end of a non-merged segment
 
         xname, yname = self.intern(intern)
+        display_iteration = logger.getEffectiveLevel() == logging.INFO
         for i_s, i_e in zip(track_s, track_e):
             if i_s == i_e or self.tracks[i_s] == self.NOGROUP:
                 continue
+            if display_iteration:
+                print(f"Network obs from {i_s} to {i_e} on {track_e[-1]}", end="\r")
             sl = slice(i_s, i_e)
             local_ids = ids[sl]
+            # built segments with local indices
             self.set_tracks(self[xname][sl], self[yname][sl], local_ids, **kwargs)
-            m = local_ids["previous_obs"] == -1
+            # shift the local indices to the total indexation for the used observations
+            m = local_ids["previous_obs"] != -1
             local_ids["previous_obs"][m] += i_s
-            m = local_ids["next_obs"] == -1
+            m = local_ids["next_obs"] != -1
             local_ids["next_obs"][m] += i_s
+        if display_iteration:
+            print()
         return ids
-        # ids_sort = ids[new_i]
-        # # To be able to follow indices sorting
-        # reverse_sort = empty(new_i.shape[0], dtype="u4")
-        # reverse_sort[new_i] = arange(new_i.shape[0])
-        # # Redirect indices
-        # m = ids_sort["next_obs"] != -1
-        # ids_sort["next_obs"][m] = reverse_sort[
-        #     ids_sort["next_obs"][m]
-        # ]
-        # m = ids_sort["previous_obs"] != -1
-        # ids_sort["previous_obs"][m] = reverse_sort[
-        #     ids_sort["previous_obs"][m]
-        # ]
-        # # print(ids_sort)
-        # display_network(
-        #     x[new_i],
-        #     y[new_i],
-        #     ids_sort["track"],
-        #     ids_sort["time"],
-        #     ids_sort["next_cost"],
-        # )
 
-    def set_tracks(self, x, y, ids, window):
+    def set_tracks(self, x, y, ids, window, **kwargs):
         """
-        Will split one group in tracks
+        Will split one group (network) in segments
 
         :param array x: coordinates of group
         :param array y: coordinates of group
@@ -662,24 +657,28 @@ class TrackEddiesObservations(EddiesObservations):
         nb = x.shape[0]
         used = zeros(nb, dtype="bool")
         track_id = 1
-        # build all polygon (need to check if wrap is needed)
-        polygons = [Polygon(create_vertice_from_2darray(x, y, i)) for i in range(nb)]
+        # build all polygons (need to check if wrap is needed)
         for i in range(nb):
-            # If observation already in one track, we go to the next one
+            # If the observation is already in one track, we go to the next one
             if used[i]:
                 continue
-            self.follow_obs(i, track_id, used, ids, polygons, *time_index, window)
+            # Search a possible continuation (forward)
+            self.follow_obs(i, track_id, used, ids, x, y, *time_index, window, **kwargs)
             track_id += 1
+            # Search a possible ancestor (backward)
+            self.get_previous_obs(i, ids, x, y, *time_index, window, **kwargs)
 
     @classmethod
-    def follow_obs(cls, i_next, track_id, used, ids, *args):
+    def follow_obs(cls, i_next, track_id, used, ids, *args, **kwargs):
+        """Associate the observations to the segments"""
+
         while i_next != -1:
             # Flag
             used[i_next] = True
             # Assign id
             ids["track"][i_next] = track_id
             # Search next
-            i_next_ = cls.next_obs(i_next, ids, *args)
+            i_next_ = cls.get_next_obs(i_next, ids, *args, **kwargs)
             if i_next_ == -1:
                 break
             ids["next_obs"][i_next] = i_next_
@@ -695,7 +694,40 @@ class TrackEddiesObservations(EddiesObservations):
             i_next = i_next_
 
     @staticmethod
-    def next_obs(i_current, ids, polygons, time_s, time_e, time_ref, window):
+    def get_previous_obs(
+        i_current, ids, x, y, time_s, time_e, time_ref, window, **kwargs
+    ):
+        """Backward association of observations to the segments"""
+
+        time_cur = ids["time"][i_current]
+        t0, t1 = time_cur - 1 - time_ref, max(time_cur - window - time_ref, 0)
+        for t_step in range(t0, t1 - 1, -1):
+            i0, i1 = time_s[t_step], time_e[t_step]
+            # No observation at the time step
+            if i0 == i1:
+                continue
+            # Search for overlaps
+            xi, yi, xj, yj = x[[i_current]], y[[i_current]], x[i0:i1], y[i0:i1]
+            ii, ij = bbox_intersection(xi, yi, xj, yj)
+            if len(ii) == 0:
+                continue
+            c = zeros(len(xj))
+            c[ij] = vertice_overlap(xi[ii], yi[ii], xj[ij], yj[ij], **kwargs)
+            # We remove low overlap
+            c[c < 0.01] = 0
+            # We get index of maximal overlap
+            i = c.argmax()
+            c_i = c[i]
+            # No overlap found
+            if c_i == 0:
+                continue
+            ids["previous_cost"][i_current] = c_i
+            ids["previous_obs"][i_current] = i0 + i
+            break
+
+    @staticmethod
+    def get_next_obs(i_current, ids, x, y, time_s, time_e, time_ref, window, **kwargs):
+        """Forward association of observations to the segments"""
         time_max = time_e.shape[0] - 1
         time_cur = ids["time"][i_current]
         t0, t1 = time_cur + 1 - time_ref, min(time_cur + window - time_ref, time_max)
@@ -706,10 +738,15 @@ class TrackEddiesObservations(EddiesObservations):
             # No observation at the time step
             if i0 == i1:
                 continue
-            # Intersection / union, to be able to separte in case of multiple inside
-            c = polygon_overlap(polygons[i_current], polygons[i0:i1])
+            # Search for overlaps
+            xi, yi, xj, yj = x[[i_current]], y[[i_current]], x[i0:i1], y[i0:i1]
+            ii, ij = bbox_intersection(xi, yi, xj, yj)
+            if len(ii) == 0:
+                continue
+            c = zeros(len(xj))
+            c[ij] = vertice_overlap(xi[ii], yi[ii], xj[ij], yj[ij], **kwargs)
             # We remove low overlap
-            c[c < 0.1] = 0
+            c[c < 0.01] = 0
             # We get index of maximal overlap
             i = c.argmax()
             c_i = c[i]
