@@ -7,10 +7,12 @@ import heapq
 
 from numba import njit, prange
 from numba import types as numba_types
-from numpy import array, concatenate, empty, nan, ones, pi, where
+from numpy import arctan, array, concatenate, empty, nan, ones, pi, where, zeros
 from numpy.linalg import lstsq
 from Polygon import Polygon
 
+from .generic import build_index
+
 
 @njit(cache=True)
 def is_left(
@@ -209,6 +211,7 @@ def winding_number_poly(x, y, xy_poly):
     # loop through all edges of the polygon
     for i_elt in range(nb_elt):
         if i_elt + 1 == nb_elt:
+            # We close polygon with first value (no need to duplicate first value)
             x_next = xy_poly[0, 0]
             y_next = xy_poly[0, 1]
         else:
@@ -321,7 +324,7 @@ def bbox_intersection(x0, y0, x1, y1):
                 continue
             i.append(i0)
             j.append(i1)
-    return array(i), array(j)
+    return array(i, dtype=numba_types.int32), array(j, dtype=numba_types.int32)
 
 
 @njit(cache=True)
@@ -452,7 +455,7 @@ def polygon_overlap(p0, p1, minimal_area=False):
 
     :param list(Polygon) p0: List of polygon to compare with p1 list
     :param list(Polygon) p1: List of polygon to compare with p0 list
-    :param bool minimal_area: If True, function will compute intersection/little polygon, else intersection/union
+    :param bool minimal_area: If True, function will compute intersection/smaller polygon, else intersection/union
     :return: Result of cost function
     :rtype: array
     """
@@ -462,15 +465,16 @@ def polygon_overlap(p0, p1, minimal_area=False):
         p_ = p1[i]
         # Area of intersection
         intersection = (p0 & p_).area()
-        # we divide intersection with the little one result from 0 to 1
+        # we divide the intersection by the smaller area, result from 0 to 1
         if minimal_area:
             cost[i] = intersection / min(p0.area(), p_.area())
-        # we divide intersection with polygon merging result from 0 to 1
+        # we divide the intersection by the merged polygons area, result from 0 to 1
         else:
             cost[i] = intersection / (p0 + p_).area()
     return cost
 
 
+# FIXME: only one function are needed
 @njit(cache=True)
 def fit_circle(x, y):
     """
@@ -517,6 +521,45 @@ def fit_circle(x, y):
     return x0, y0, radius, err
 
 
+@njit(cache=True)
+def fit_ellips(x, y):
+    r"""
+    From a polygon, function will fit an ellips.
+
+    Must be call with local coordinates (in m, to get a radius in m).
+
+    .. math:: (\frac{x - x_0}{a})^2 + (\frac{y - y_0}{b})^2 = 1
+
+    .. math:: (\frac{x^2 - 2 * x * x_0 + x_0 ^2}{a^2}) + \frac{y^2 - 2 * y * y_0 + y_0 ^2}{b^2}) = 1
+
+    In case of angle
+    https://en.wikipedia.org/wiki/Ellipse
+
+    """
+    # x,y = x[1:],y[1:]
+    nb = x.shape[0]
+    datas = ones((nb, 5), dtype=x.dtype)
+    datas[:, 0] = x ** 2
+    datas[:, 1] = x * y
+    datas[:, 2] = y ** 2
+    datas[:, 3] = x
+    datas[:, 4] = y
+    (a, b, c, d, e), _, _, _ = lstsq(datas, ones(nb, dtype=x.dtype))
+    det = b ** 2 - 4 * a * c
+    if det > 0:
+        print(det)
+    x0 = (2 * c * d - b * e) / det
+    y0 = (2 * a * e - b * d) / det
+
+    AB1 = 2 * (a * e ** 2 + c * d ** 2 - b * d * e - det)
+    AB2 = a + c
+    AB3 = ((a - c) ** 2 + b ** 2) ** 0.5
+    A = -((AB1 * (AB2 + AB3)) ** 0.5) / det
+    B = -((AB1 * (AB2 - AB3)) ** 0.5) / det
+    theta = arctan((c - a - AB3) / b)
+    return x0, y0, A, B, theta
+
+
 @njit(cache=True)
 def fit_circle_(x, y):
     r"""
@@ -671,12 +714,12 @@ def tri_area2(x, y, i0, i1, i2):
 
 
 @njit(cache=True)
-def visvalingam(x, y, nb_pt=18):
+def visvalingam(x, y, fixed_size=18):
     """Polygon simplification with visvalingam algorithm
 
     :param array x:
     :param array y:
-    :param int nb_pt: array size of out
+    :param int fixed_size: array size of out
     :return: New (x, y) array
     :rtype: array,array
     """
@@ -696,7 +739,7 @@ def visvalingam(x, y, nb_pt=18):
         i0 = i1
         i1 = i
     # we continue until we are equal to nb_pt
-    while len(h) >= nb_pt:
+    while len(h) >= fixed_size:
         # We pop lower area
         _, (i0, i1, i2) = heapq.heappop(h)
         # We check if triangle is valid(i0 or i2 not removed)
@@ -714,13 +757,174 @@ def visvalingam(x, y, nb_pt=18):
             # in this case we replace two point
             i0, i2 = i_p, i_n
         heapq.heappush(h, (tri_area2(x, y, i0, i1, i2), (i0, i1, i2)))
-    x_new, y_new = empty(nb_pt, dtype=x.dtype), empty(nb_pt, dtype=y.dtype)
+    x_new, y_new = empty(fixed_size, dtype=x.dtype), empty(fixed_size, dtype=y.dtype)
     j = 0
     for i, i_n in enumerate(i_next):
         if i_n == -1:
             x_new[j] = x[i]
             y_new[j] = y[i]
             j += 1
-    x_new[j] = x_new[0]
-    y_new[j] = y_new[0]
+    # we copy first value to fill array end
+    x_new[j:] = x_new[0]
+    y_new[j:] = y_new[0]
     return x_new, y_new
+
+
+@njit(cache=True)
+def reduce_size(x, y):
+    """
+    Reduce array size if last position is repeated, in order to save compute time
+
+    :param array x: longitude
+    :param array y: latitude
+
+    :return: reduce arrays x,y
+    :rtype: ndarray,ndarray
+    """
+    i = x.shape[0]
+    x0, y0 = x[0], y[0]
+    while True:
+        i -= 1
+        if x[i] != x0 or y[i] != y0:
+            i += 1
+            # In case of virtual obs all value could be fill with same value, to avoid empty array
+            i = max(3, i)
+            return x[:i], y[:i]
+
+
+@njit(cache=True)
+def group_obs(x, y, step, nb_x):
+    """Get index k_box for each box, and indexes to sort"""
+    nb = x.size
+    i = empty(nb, dtype=numba_types.uint32)
+    for k in range(nb):
+        i[k] = box_index(x[k], y[k], step, nb_x)
+    return i, i.argsort(kind="mergesort")
+
+
+@njit(cache=True)
+def box_index(x, y, step, nb_x):
+    """Return k_box index for each value"""
+    return numba_types.uint32((x % 360) // step + nb_x * ((y + 90) // step))
+
+
+@njit(cache=True)
+def box_indexes(x, y, step):
+    """Return i_box,j_box index for each value"""
+    return numba_types.uint32((x % 360) // step), numba_types.uint32((y + 90) // step)
+
+
+@njit(cache=True)
+def poly_indexs_(x_p, y_p, x_c, y_c):
+    """
+    Index of contour for each postion inside a contour, -1 in case of no contour
+
+    :param array x_p: longitude to test (must be define, no nan)
+    :param array y_p: latitude to test (must be define, no nan)
+    :param array x_c: longitude of contours
+    :param array y_c: latitude of contours
+    """
+    nb_x = 360
+    step = 1.0
+    i, i_order = group_obs(x_p, y_p, step, nb_x)
+    nb_p = x_p.shape[0]
+    nb_c = x_c.shape[0]
+    indexs = -ones(nb_p, dtype=numba_types.int32)
+    # Adress table to get particle bloc
+    start_index, end_index, i_first = build_index(i[i_order])
+    nb_bloc = end_index.size
+    for i_contour in range(nb_c):
+        # Build vertice and box included contour
+        x_, y_ = reduce_size(x_c[i_contour], y_c[i_contour])
+        x_c_min, y_c_min = x_.min(), y_.min()
+        x_c_max, y_c_max = x_.max(), y_.max()
+        v = create_vertice(x_, y_)
+        i0, j0 = box_indexes(x_c_min, y_c_min, step)
+        i1, j1 = box_indexes(x_c_max, y_c_max, step)
+        # i0 could be greater than i1, (x_c is always continious) so you could have a contour over bound
+        if i0 > i1:
+            i1 += nb_x
+        for i_x in range(i0, i1 + 1):
+            # we force i_x in 0 360 range
+            i_x %= nb_x
+            for i_y in range(j0, j1 + 1):
+                # Get box indices
+                i_box = i_x + nb_x * i_y - i_first
+                # Indice must be in table range
+                if i_box < 0 or i_box >= nb_bloc:
+                    continue
+                for i_p_ordered in range(start_index[i_box], end_index[i_box]):
+                    i_p = i_order[i_p_ordered]
+                    if indexs[i_p] != -1:
+                        continue
+                    y = y_p[i_p]
+                    if y > y_c_max:
+                        continue
+                    if y < y_c_min:
+                        continue
+                    # Normalize longitude at +-180° around x_c_min
+                    x = (x_p[i_p] - x_c_min + 180) % 360 + x_c_min - 180
+                    if x > x_c_max:
+                        continue
+                    if x < x_c_min:
+                        continue
+                    if winding_number_poly(x, y, v) != 0:
+                        indexs[i_p] = i_contour
+    return indexs
+
+
+@njit(cache=True)
+def poly_indexs(x_p, y_p, x_c, y_c):
+    """
+    index of contour for each postion inside a contour, -1 in case of no contour
+
+    :param array x_p: longitude to test
+    :param array y_p: latitude to test
+    :param array x_c: longitude of contours
+    :param array y_c: latitude of contours
+    """
+    nb_p = x_p.shape[0]
+    nb_c = x_c.shape[0]
+    indexs = -ones(nb_p, dtype=numba_types.int32)
+    for i in range(nb_c):
+        x_, y_ = reduce_size(x_c[i], y_c[i])
+        x_c_min, y_c_min = x_.min(), y_.min()
+        x_c_max, y_c_max = x_.max(), y_.max()
+        v = create_vertice(x_, y_)
+        for j in range(nb_p):
+            if indexs[j] != -1:
+                continue
+            x, y = x_p[j], y_p[j]
+            if x > x_c_min and x < x_c_max and y > y_c_min and y < y_c_max:
+                if winding_number_poly(x, y, v) != 0:
+                    indexs[j] = i
+    return indexs
+
+
+@njit(cache=True)
+def insidepoly(x_p, y_p, x_c, y_c):
+    """
+    True for each postion inside a contour
+
+    :param array x_p: longitude to test
+    :param array y_p: latitude to test
+    :param array x_c: longitude of contours
+    :param array y_c: latitude of contours
+    """
+    # TODO must be optimize like poly_index
+    nb_p = x_p.shape[0]
+    nb_c = x_c.shape[0]
+    flag = zeros(nb_p, dtype=numba_types.bool_)
+    for i in range(nb_c):
+        x_, y_ = reduce_size(x_c[i], y_c[i])
+        x_c_min, y_c_min = x_.min(), y_.min()
+        x_c_max, y_c_max = x_.max(), y_.max()
+        v = create_vertice(x_, y_)
+        for j in range(nb_p):
+            if flag[j]:
+                continue
+            x, y = x_p[j], y_p[j]
+            if x > x_c_min and x < x_c_max and y > y_c_min and y < y_c_max:
+                if winding_number_poly(x, y, v) != 0:
+                    flag[j] = True
+    return flag
